#include "xparameters.h"    /* generated system parameters                  */
#include "xbasic_types.h"   /* Xilinx basic types for device drivers        */
#include "xtmrctr.h"        /* layer 1 timer counter device driver          */
#include "xintc.h"          /* layer 1 interrupt controller device driver */
#include "xgpio.h"     /* layer 0/1 GPIO device driver                 */

#define printf xil_printf   /* a smaller footprint printf */
#define RIGHT   0x1
#define LEFT    0x2
#define UP      0x8
#define DOWN    0x4
#define B1      0x16


void ppu_write(int addr, unsigned char data)
{
	volatile unsigned char* real_addr = XPAR_PPUBRIDGE_0_BASEADDR + (unsigned char*)addr;
	
	*real_addr = data;
}
unsigned char ppu_read(int addr)
{
	volatile unsigned char* real_addr = XPAR_PPUBRIDGE_0_BASEADDR + (unsigned char*)addr;
	
	return *real_addr;
}

void wait_for_VBlank();
void wait_for_X_VBlank(int count_VBlank);
void wait_for_Timer();
void wait_for_Release_VBlank();
void PPUVsyncHandler();
void TimerHandler();
void int_handler(void* arg);
void scrolling(int x, int y);


int counter_value = 0,counter=0, counter_timer=0;
int interrupt_flag=0,interrupt_flag_1=0;
unsigned char var1, var2, var3;

//------------------------------------------------------------------
//------------              Graphical objects           ------------
//------------                motor cycle               ------------
unsigned char Sprite_Set[6][3];
void move_sprite(int spr_num,int x_position, int y_position);
//------------------------------------------------------------------


typedef struct {
	Xuint32 Blah       ;
	Xuint32 BaseAddress;
} PPUBr;
PPUBr PPUBridge; 

XIntc InterruptController;
XGpio Gpio;

int main()
{
  int i,k,m;
  unsigned char c;
  Xuint32 buttons;
  
  printf("\n\rStarting the Application\n\r");

  //--------------------------------------------------------------------------//
  //----------------------- Interrupt Controller Setup -----------------------//  
  unsigned char *ptr = XPAR_PPUBRIDGE_0_BASEADDR;                             
  XStatus Status;
  PPUBridge.BaseAddress = XPAR_PPUBRIDGE_0_BASEADDR;
  Status = XIntc_Initialize(&InterruptController, XPAR_OPB_INTC_0_DEVICE_ID);
  if (Status != XST_SUCCESS)
  {
    printf("Interrupt controller initialization error\n\r");
  }else{
	 printf("Interrupt controller initialized fine\n\r");
  }
  Status = XIntc_Connect(&InterruptController,
                          XPAR_OPB_INTC_0_PPUBRIDGE_0_IP2INTC_IRPT_INTR,
                        (XInterruptHandler)PPUVsyncHandler,
                         &PPUBridge);
  Status = XIntc_Connect(&InterruptController,
                          XPAR_OPB_INTC_0_OPB_TIMER_0_INTERRUPT_INTR,
                        (XInterruptHandler)TimerHandler,
                         &PPUBridge);
  if (Status != XST_SUCCESS)
  {
      printf("Interrupt controller connect error\n\r");
  }
  Status = XIntc_Start(&InterruptController, XIN_REAL_MODE);
  if (Status != XST_SUCCESS)
  {
      printf("Interrupt controller error\n\r");
  }
  XIntc_Enable(&InterruptController,
               XPAR_OPB_INTC_0_PPUBRIDGE_0_IP2INTC_IRPT_INTR);
  XIntc_Enable(&InterruptController,
               XPAR_OPB_INTC_0_OPB_TIMER_0_INTERRUPT_INTR);					 
  microblaze_enable_interrupts();
  //------------------------------------------------------------------------//
  
   Status = XGpio_Initialize(&Gpio, XPAR_OPB_GPIO_0_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        printf("GPIO initialization error\n\r\r");
    }
    /* Set the direction for all signals to be inputs
     */
    XGpio_SetDataDirection(&Gpio, 1, ~0x0);

	 
	 
  //------------------------------------------------------------------------//
  //------------------        Initializing the PPU 	        --------------//
  printf("Initializing PPU \n\r");	 
  printf("........... \n\r");
  ppu_write(0, 0x90);//10010000 - Enable NMI-X-8*8sprite-Backg Patt.-Spr. 
	                  //Patt.-Increment addr 1- Name Table00-
  ppu_write(1, 0x00);//00000000 - Backg Color Den.-show sprite-show backg-
                     //no sprite clip-clipping backg-color mon.
  ppu_read(2);//PPU Status Register
  //blank name table
  
  //ppu_write(6, 0x00);
  //ppu_write(6, 0x00);
  //ppu_write(7, 0x00);
  
  ppu_write(6, 0x20);
  ppu_write(6, 0x00);
  for(i=0; i<960; i++)
      ppu_write(7, 0xFC);

  ppu_write(3, 0x00);
  for(i =0; i<64; i++){
      ppu_write(4, 0xEF);
      ppu_write(4, 0x00);
      ppu_write(4, 0x10);
      ppu_write(4, 0x00);
  }

  ppu_read(2);
  
  ppu_write(3, 0x04);
  ppu_write(4, 0x96);
  ppu_write(4, 0x09);
  ppu_write(4, 0x00);
  ppu_write(4, 0x80);


  //------------------------------------------------------------------------//

  //------------------------------------------------------------------------//
  //-----------------             movie here              ------------------//
  //------------------------------------------------------------------------//
  ppu_write(1, 0x18);

for(i=0; i<1000000; i++);
  ppu_write(1, 0x18);
  ppu_write(3, 0x06);
  ppu_write(4, 0x80);//vertical mirroring
  
for(i=0; i<10000000; i++);
  ppu_write(1, 0x18);
  ppu_write(3, 0x06);
  ppu_write(4, 0xC0);//both mirroring
 
  /* ppu_write(1, 0x18);
  ppu_write(3, 0x06);
  ppu_write(4, 0x01);
  wait_for_X_VBlank(100);
    ppu_write(1, 0x18);
  ppu_write(3, 0x06);
  ppu_write(4, 0x02);
  wait_for_X_VBlank(100);
    ppu_write(1, 0x18);
  ppu_write(3, 0x06);
  ppu_write(4, 0x03);
  wait_for_X_VBlank(100);
*/

  //for (i=0; i<5000; i++)
  //{
      //buttons = XGpio_DiscreteRead(&Gpio,1);	 

      //wait_for_VBlank() ;
      //ppu_read(2);

      /*if((buttons & RIGHT) == 0){
	x_pos++;
	} else if((buttons & LEFT) == 0){
	x_pos--;
	} else if((buttons & UP) == 0 && y_pos > 90){
	y_pos--;
	} else if((buttons & DOWN) == 0&& y_pos < 150){
	y_pos++;
	} */


      //move_sprite(29, 8+x_pos-i,16+y_pos);

      //wait_for_X_VBlank(10);
  //}

  //------------------------------------------------------------------------//

  wait_for_Release_VBlank()            ;
  printf("Interrupt finished ....\n\r");

  ppu_write(6, 0x00)                   ;
  ppu_write(6, (unsigned char)  0)     ;
	
  //screen scrolling;
  printf("Scrolling ....\n\r");
  scrolling(13,0);
  
  //ppu_write(1, 0x18)                 ;
  printf("Exiting the Application\n\r");
}

void PPUVsyncHandler(void *InstancePtr){
  if(interrupt_flag==0)
  {
	 counter++;
	 interrupt_flag=1;

  }
}

void TimerHandler(void *InstancePtr){
  if(interrupt_flag_1==0)
  {
	 interrupt_flag_1=1;
  }
  counter_timer++;
}

void wait_for_Release_VBlank()
{
  int flag=0;
  unsigned char c;
  while (flag==0)
  {
    c=ppu_read(2);
	 c=c & 0x80;
	 if (c==0x00)
	 {
	   flag=1;
	 }
  }
}

void wait_for_VBlank()
{
  while(interrupt_flag==0){
  }
  interrupt_flag=0;
}

void wait_for_X_VBlank(int count_VBlank)
{
  counter=0;
  while (counter<count_VBlank)
  {
    while(interrupt_flag==0){
    }
    interrupt_flag=0;
  }
}

void wait_for_Timer()
{
  while(interrupt_flag_1==0){
  }
  interrupt_flag_1=0;
}

void scrolling(int x, int y)
{
  int i;
  wait_for_VBlank()              ;
  //ppu_write(1, 0x00)             ;
  ppu_write(5, (unsigned char) x);
  ppu_write(5, (unsigned char) y);
  //ppu_write(1, 0x18)             ;
  
  for(i=0; i<23000; i++);
  
  //ppu_write(6, (unsigned char) 0);
  ppu_write(5, (unsigned char) 0);
    ppu_write(5, (unsigned char) 0);
  //ppu_write(6, (unsigned char) 0);
  
    /*for(i=0; i<20000; i++);
  
  ppu_write(5, (unsigned char) x);
  ppu_write(5, (unsigned char) y);*/
  
}

void move_sprite(int spr_num,int x_position, int y_position)
{
	  //ppu_write(1, 0x00);
	  //ppu_read(2)       ;
	  //wait_for_VBlank() ;
	  
	  ppu_write(3,(unsigned char) 0x00)       ; 
	  ppu_write(3,(unsigned char) spr_num*4)  ;  
	  ppu_write(4,(unsigned char) y_position) ; //set the y-position
	    
	  //ppu_write(3,(unsigned char) 0x00)       ; 
	  //ppu_write(3,(unsigned char) spr_num*4+1); 
	  //c=ppu_read(4)                           ; //set the tile number
	   
     ppu_write(3,(unsigned char) 0x00)       ; 
     ppu_write(3,(unsigned char) spr_num*4+3);  
     ppu_write(4,(unsigned char) x_position) ; //set the x-position
	  
	  //ppu_write(1, 0x18)                            ;
}
