`timescale 1ns / 1ps

module bg_buffer
(
	input i_clk,
	input i_reset,
	
	input [7:0] i_pt0,
	input [7:0] i_pt1,
	input [1:0] i_at,
	input [2:0] i_fh,
	input i_load,
	input i_shift_en,
	
	output reg [3:0] o_pixel
);

reg [15:0] sr_pt0;
reg [15:0] sr_pt1;
reg [8:0] sr_at0;
reg [8:0] sr_at1;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		sr_pt0 <= 'd0;
		sr_pt1 <= 'd0;
		sr_at0 <= 'd0;
		sr_at1 <= 'd0;
	end
	else begin
		if (i_load) begin
			sr_pt0[7:0] <= i_pt0;
			sr_pt1[7:0] <= i_pt1;
			sr_at0[0] <= i_at[0];
			sr_at1[0] <= i_at[1];
		end
		else if (i_shift_en) begin
			sr_pt0[7:0] <= {sr_pt0[6:0], 1'b0};
			sr_pt1[7:0] <= {sr_pt1[6:0], 1'b0};
		end
		
		if (i_shift_en) begin
			sr_pt0[15:8] <= sr_pt0[14:7];
			sr_pt1[15:8] <= sr_pt1[14:7];
			sr_at0[8:1] <= sr_at0[7:0];
			sr_at1[8:1] <= sr_at1[7:0];
		end
	end
end

always @* begin
	case (i_fh)
		3'd0 : o_pixel = {sr_at1[8], sr_at0[8], sr_pt1[15], sr_pt0[15]};
		3'd1 : o_pixel = {sr_at1[7], sr_at0[7], sr_pt1[14], sr_pt0[14]};
		3'd2 : o_pixel = {sr_at1[6], sr_at0[6], sr_pt1[13], sr_pt0[13]};
		3'd3 : o_pixel = {sr_at1[5], sr_at0[5], sr_pt1[12], sr_pt0[12]};
		3'd4 : o_pixel = {sr_at1[4], sr_at0[4], sr_pt1[11], sr_pt0[11]};
		3'd5 : o_pixel = {sr_at1[3], sr_at0[3], sr_pt1[10], sr_pt0[10]};
		3'd6 : o_pixel = {sr_at1[2], sr_at0[2], sr_pt1[9], sr_pt0[9]};
		3'd7 : o_pixel = {sr_at1[1], sr_at0[1], sr_pt1[8], sr_pt0[8]};
	endcase
end


endmodule
