`timescale 1ns / 1ps

module bg_fetch
(
	input i_clk,
	input i_reset,

	input i_go,
	
	output reg o_hscroll_cnt_enable,
	input [7:0] i_vram_data,
	
	output reg o_nt_read,
	output reg o_pt_read,
	output reg o_at_read,
	output reg o_pt_read_bit3,
	
	output reg o_bg_valid,
	output reg [7:0] o_bg_pt0,
	output reg [7:0] o_bg_pt1
);

reg [7:0] pt0;

localparam S_IDLE = 4'd0;
localparam S_NT = 4'd1;
localparam S_AT = 4'd4;
localparam S_PT0 = 4'd2;
localparam S_PT1 = 4'd3;
localparam S_WAIT0 = 4'd5;
localparam S_WAIT1 = 4'd6;
localparam S_WAIT2 = 4'd7;
localparam S_WAIT3 = 4'd8;

reg [3:0] state;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		state <= S_IDLE;
	end
	else begin
		case (state)
			S_IDLE: begin
				if (i_go) begin
					state <= S_NT;
				end
			end
			
			S_NT: state <= S_WAIT0;
			S_WAIT0: state <= S_AT;
			
			S_AT: state <= S_WAIT1;
			S_WAIT1: state <= S_PT0;
			
			S_PT0: state <= S_WAIT2;
			S_WAIT2: begin
				pt0 <= i_vram_data;
				state <= S_PT1;
			end
			
			S_PT1: state <= S_WAIT3;
			S_WAIT3: begin			
				if (i_go) begin
					state <= S_NT;
				end
				else begin
					state <= S_IDLE;
				end
			end
			
			default: state <= S_IDLE;
		endcase
	end
end

always @* begin
	o_nt_read = 1'b0;
	o_pt_read = 1'b0;
	o_at_read = 1'b0;
	o_pt_read_bit3 = 1'bx;
	o_bg_pt0 = 8'bx;
	o_bg_pt1 = 8'bx;
	o_bg_valid = 1'b0;
	o_hscroll_cnt_enable = 1'b0;
	
	case (state)
		S_NT: o_nt_read = 1'b1;
		
		S_AT: o_at_read = 1'b1;
		
		S_PT0: begin
			o_pt_read = 1'b1;
			o_pt_read_bit3 = 1'b0;
		end
		
		S_PT1: begin
			o_pt_read = 1'b1;
			o_pt_read_bit3 = 1'b1;
		end
		
		S_WAIT3: begin
			o_bg_valid = 1'b1;
			o_bg_pt0 = pt0;
			o_bg_pt1 = i_vram_data;
			o_hscroll_cnt_enable = 1'b1;
		end
	endcase
end

endmodule
