`timescale 1ns / 1ps

module in_range_evaluate
(
	input i_clk,
	input i_reset,
	
	input i_enable,
	
	output [7:0] o_spr_ram_addr,
	input [7:0] i_spr_ram_data,
	
	output reg o_spr_datavalid,
	output [7:0] o_spr_x,
	output [7:0] o_spr_y,
	output [7:0] o_spr_tilenum,
	output [7:0] o_spr_attr,
	output reg [5:0] o_spr_index
);

wire addr_counter_en;
wire addr_counter_rst;
reg [8:0] addr_counter;

assign addr_counter_en = !addr_counter[8] && i_enable;
assign addr_counter_rst = addr_counter[8];

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		addr_counter <= 'd0;
	end
	else begin
		if (addr_counter_rst) begin
			addr_counter <= 'd0;
		end
		else if (addr_counter_en) begin
			addr_counter <= addr_counter + 9'd1;
		end
	end
end

reg [7:0] ycoord;
reg [7:0] tilenum;
reg [7:0] attr;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		o_spr_datavalid <= 'd0;
	end
	else begin
		case (addr_counter[1:0])
			2'd1 : ycoord <= i_spr_ram_data;
			2'd2 : tilenum <= i_spr_ram_data;
			2'd3 : attr <= i_spr_ram_data;
		endcase
		
		o_spr_datavalid <= addr_counter[1:0] == 2'd3;
		o_spr_index <= addr_counter[7:2];
	end
end


assign o_spr_ram_addr = addr_counter[7:0];

assign o_spr_y = ycoord;
assign o_spr_tilenum = tilenum;
assign o_spr_attr = attr;
assign o_spr_x = i_spr_ram_data;





endmodule
