`timescale 1ns / 1ps

module ppu
(
	input i_clk_54,
	input i_clk_17,
	input i_reset,
	
	input [2:0] i_addr,
	input [7:0] i_wr_data,
	output [7:0] o_rd_data,
	input i_read,
	input i_write,
	output o_nmi,
	
	output [5:0] o_pixel,
	output o_pixel_valid,
	output [7:0] o_pixel_x,
	output [7:0] o_pixel_y
);

wire [8:0] rend_cur_x;
wire [7:0] rend_cur_y;

wire [2:0] cpu_addr;
wire [7:0] cpu_wr_data;
wire [7:0] cpu_rd_data;
wire cpu_read;
wire cpu_write;
wire cpu_rd_datavalid;
nes_iface nes_iface_inst
(
	.i_clk_54(i_clk_54),
	.i_clk_17(i_clk_17),
	.i_reset(i_reset),
	
	.i_nes_addr(i_addr),
	.i_nes_wr_data(i_wr_data),
	.o_nes_rd_data(o_rd_data),
	.i_nes_write(i_write),
	.i_nes_read(i_read),
	
	.o_addr(cpu_addr),
	.o_wr_data(cpu_wr_data),
	.i_rd_data(cpu_rd_data),
	.o_write(cpu_write),
	.o_read(cpu_read),
	.i_rd_datavalid(cpu_rd_datavalid)
);

wire [13:0] vram_addr;
wire vram_read;
wire vram_write;
wire [7:0] vram_read_data;
wire [7:0] vram_write_data;
wire [4:0] pal_addr_cpu;
wire [5:0] pal_rd_data_cpu;
wire [5:0] pal_wr_data_cpu;
wire pal_write_cpu;
wire [7:0] spr_addr_cpu;
wire [7:0] spr_rd_data_cpu;
wire [7:0] spr_wr_data_cpu;
wire spr_write_cpu;
wire hscroll_cnt_load;
wire all_cnt_load;
wire hscroll_cnt_enable;
wire vscroll_cnt_enable;
wire rend_nt_read;
wire rend_at_read;
wire rend_bpt_read;
wire rend_bpt_bit3;
wire rend_opt_read;
wire rend_opt_bit3;
wire rend_opt_ptab;
wire [2:0] rend_opt_fv;
wire [7:0] rend_opt_tilenum;
wire [1:0] rend_attrib;
wire [2:0] fine_h;
wire rend_set_vblank;
wire rend_clear_vblank;
wire rend_spritelimit;
wire rend_primaryhit;
wire object_pat_table;
wire object_big;
wire object_clip;
wire object_enable;
wire pf_clip;
wire pf_enable;

regs regs_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),
	
	.o_vblank_nmi(o_nmi),
	
	.i_cpu_addr(cpu_addr),
	.i_cpu_write(cpu_write),
	.i_cpu_read(cpu_read),
	.i_cpu_write_data(cpu_wr_data),
	.o_cpu_read_data(cpu_rd_data),
	.o_cpu_datavalid(cpu_rd_datavalid),
	
	.o_vram_addr(vram_addr),
	.o_vram_read(vram_read),
	.o_vram_write(vram_write),
	.i_vram_read_data(vram_read_data),
	.o_vram_write_data(vram_write_data),
	
	.o_pal_addr(pal_addr_cpu),
	.o_pal_wr_data(pal_wr_data_cpu),
	.o_pal_write(pal_write_cpu),
	.i_pal_rd_data(pal_rd_data_cpu),
	
	.o_spr_addr(spr_addr_cpu),
	.o_spr_wr_data(spr_wr_data_cpu),
	.o_spr_write(spr_write_cpu),
	.i_spr_rd_data(spr_rd_data_cpu),
	
	.i_hscroll_cnt_load(hscroll_cnt_load),
	.i_all_cnt_load(all_cnt_load),
	.i_hscroll_cnt_enable(hscroll_cnt_enable),
	.i_vscroll_cnt_enable(vscroll_cnt_enable),
	.i_rend_nt_read(rend_nt_read),
	.i_rend_at_read(rend_at_read),
	.i_rend_bpt_read(rend_bpt_read),
	.i_rend_bpt_bit3(rend_bpt_bit3),
	.i_rend_opt_read(rend_opt_read),
	.i_rend_opt_bit3(rend_opt_bit3),
	.i_rend_opt_ptab(rend_opt_ptab),
	.i_rend_opt_fv(rend_opt_fv),
	.i_rend_opt_tilenum(rend_opt_tilenum),
	.o_attrib(rend_attrib),
	.o_fine_h(fine_h),
	.i_rend_set_vblank(rend_set_vblank),
	.i_rend_clear_vblank(rend_clear_vblank),
	.i_rend_spritelimit(rend_spritelimit),
	.i_rend_primaryhit(rend_primaryhit),
	.o_object_pat_table(object_pat_table),
	.o_object_big(object_big),
	.o_object_clip(object_clip),
	.o_object_enable(object_enable),
	.o_pf_clip(pf_clip),
	.o_pf_enable(pf_enable)
);

wire [7:0] bg_pt0;
wire [7:0] bg_pt1;
wire bgbuf_load;
wire bgbuf_enable;
wire [3:0] bgbuf_pixel;
bg_buffer bg_buffer_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),
	
	.i_pt0(bg_pt0),
	.i_pt1(bg_pt1),
	.i_at(rend_attrib),
	.i_fh(fine_h),
	.i_load(bgbuf_load),
	.i_shift_en(bgbuf_enable),
	
	.o_pixel(bgbuf_pixel)
);

wire bgfetch_go;
bg_fetch bg_fetch_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),

   .i_go(bgfetch_go),
	
	.o_hscroll_cnt_enable(hscroll_cnt_enable),
	.i_vram_data(vram_read_data),
	
	.o_nt_read(rend_nt_read),
	.o_pt_read(rend_bpt_read),
	.o_at_read(rend_at_read),
	.o_pt_read_bit3(rend_bpt_bit3),
	
	.o_bg_valid(bgbuf_load),
	.o_bg_pt0(bg_pt0),
	.o_bg_pt1(bg_pt1)
);

wire ire_enable;
wire [7:0] spr_addr_ire;
wire [7:0] spr_data_ire;
wire ire_datavalid;
wire [7:0] ire_spr_x;
wire [7:0] ire_spr_y;
wire [7:0] ire_spr_tilenum;
wire [7:0] ire_spr_attr;
wire [5:0] ire_spr_index;
in_range_evaluate ire_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),
	
	.i_enable(ire_enable),
	
	.o_spr_ram_addr(spr_addr_ire),
	.i_spr_ram_data(spr_data_ire),
	
	.o_spr_datavalid(ire_datavalid),
	.o_spr_x(ire_spr_x),
	.o_spr_y(ire_spr_y),
	.o_spr_tilenum(ire_spr_tilenum),
	.o_spr_attr(ire_spr_attr),
	.o_spr_index(ire_spr_index)
);

wire [2:0] tmpmem_addrA;
wire [24:0] tmpmem_dataA;
wire tmpmem_writeA;
wire [2:0] tmpmem_addrB;
wire [24:0] tmpmem_dataB;
spr_tmpmem tmpmem_inst
(
	.i_clk(i_clk_54),
	
	.i_addrA(tmpmem_addrA),
	.i_dataA(tmpmem_dataA),
	.i_writeA(tmpmem_writeA),
	
	.i_addrB(tmpmem_addrB),
	.o_dataB(tmpmem_dataB)
);

wire spreval_cnt_reset;
wire [3:0] spreval_num_sprites;
spr_evaluator spr_eval_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),
	
	.i_data_valid(ire_datavalid),
	.i_cur_y(rend_cur_y),
	.i_spr_x_coord(ire_spr_x),	
	.i_spr_y_coord(ire_spr_y),
	.i_spr_tilenum(ire_spr_tilenum),
	.i_spr_attr(ire_spr_attr),
	.i_spr_index(ire_spr_index),
	
	.i_cnt_reset(spreval_cnt_reset),
	.o_num_sprites(spreval_num_sprites),
	
	.o_tmpmem_write(tmpmem_writeA),
	.o_tmpmem_addr(tmpmem_addrA),
	.o_tmpmem_data(tmpmem_dataA),
	
	.i_object_big(object_big),
	.o_set_spritelimit(rend_spritelimit)
);

wire [4:0] pal_addr_rend;
wire [5:0] pal_data_rend;
palette pal_inst
(
	.i_clk(i_clk_54),
	
	.i_cpu_addr(pal_addr_cpu),
	.i_cpu_wr_data(pal_wr_data_cpu),
	.i_cpu_write(pal_write_cpu),
	.o_cpu_rd_data(pal_rd_data_cpu),
	
	.i_rend_addr(pal_addr_rend),
	.o_rend_data(pal_data_rend)
);

wire [7:0] sprbuf_load;
wire sprbuf_enable;
wire [7:0] sprbuf_pt0;
wire [7:0] sprbuf_pt1;
wire [1:0] sprbuf_pal;
wire [7:0] sprbuf_x;
wire sprbuf_bgpri;
wire sprbuf_spr0;
wire [3:0] spr_pixel;
wire spr_bgpri;
wire spr_spr0;
spr_buffers spr_buffers_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),
	.i_load(sprbuf_load),
	.i_enable(sprbuf_enable),
	.i_pt0(sprbuf_pt0),
	.i_pt1(sprbuf_pt1),
	.i_pal(sprbuf_pal),
	.i_sprite_x(sprbuf_x),
	.i_bgpri(sprbuf_bgpri),
	.i_spr0(sprbuf_spr0),
	.o_pixel(spr_pixel),
	.o_bgpri(spr_bgpri),
	.o_spr0(spr_spr0)
);

wire spr_fetch_go;
spr_fetcher spr_fetcher_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),
	
	.o_tmpmem_addr(tmpmem_addrB),
	.i_tmpmem_data(tmpmem_dataB),
	
	.o_pt_read(rend_opt_read),
	.o_pt_bit3(rend_opt_bit3),
	.o_pt_ptab(rend_opt_ptab),
	.o_pt_fv(rend_opt_fv),
	.o_pt_tilenum(rend_opt_tilenum),
	.i_pt_data(vram_read_data),
	
	.i_go(spr_fetch_go),
	.i_num_sprites(spreval_num_sprites),
	.i_object_big(object_big),
	.i_object_ptab(object_pat_table),
	
	.o_sprbuf_load(sprbuf_load),
	.o_sprbuf_p0(sprbuf_pt0),
	.o_sprbuf_p1(sprbuf_pt1),
	.o_sprbuf_x(sprbuf_x),
	.o_sprbuf_attr({sprbuf_spr0, sprbuf_bgpri, sprbuf_pal})
);

spr_ram spr_ram_inst
(
	.i_clk(i_clk_54),
	.i_cpu_addr(spr_addr_cpu),
	.i_cpu_write(spr_write_cpu),
	.i_cpu_wr_data(spr_wr_data_cpu),
	.o_cpu_rd_data(spr_rd_data_cpu),
	.i_rend_addr(spr_addr_ire),
	.o_rend_data(spr_data_ire)
);

vidmem vidmem_inst
(
	.i_clk(i_clk_54),
	.i_addr(vram_addr),
	.i_wr_data(vram_write_data),
	.i_write(vram_write),
	.i_read(vram_read),
	.o_rd_data(vram_read_data)
);

main main_inst
(
	.i_clk(i_clk_54),
	.i_reset(i_reset),
	
	.o_bg_buf_shift_en(bgbuf_enable),
	.o_bg_fetch_go(bgfetch_go),
	.o_ire_enable(ire_enable),
	.o_spr_eval_reset(spreval_cnt_reset),
	.o_spr_buf_enable(sprbuf_enable),
	.o_spr_fetch_go(spr_fetch_go),
	.o_memcnt_loadall(all_cnt_load),
	.o_memcnt_loadh(hscroll_cnt_load),
	.o_memcnt_incv(vscroll_cnt_enable),
	.o_pixel_valid(o_pixel_valid),
	
	.o_cur_x(rend_cur_x),
	.o_cur_y(rend_cur_y),
	
	.o_set_vblank(rend_set_vblank),
	.o_clear_vblank(rend_clear_vblank),
	.i_object_clip(object_clip),
	.i_object_enable(object_enable),
	.i_pf_clip(pf_clip),
	.i_pf_enable(pf_enable)
);


wire [3:0] final_pixel;
wire final_pal;
composite_mux comp_mux_inst
(
	.i_spr_pix(spr_pixel),
	.i_spr_bgpri(spr_bgpri),
	.i_spr_spr0(spr_spr0),
	.i_bg_pix(bgbuf_pixel),
	.o_pix(final_pixel),
	.o_use_spr_pal(final_pal),
	.o_spr0_hit(rend_primaryhit)
);

assign pal_addr_rend = {final_pal, final_pixel};
assign o_pixel = pal_data_rend;
assign o_pixel_x = rend_cur_x[7:0];
assign o_pixel_y = rend_cur_y;

endmodule
