`timescale 1ns / 1ps

module ppubridge #
(
	parameter C_BASEADDR = 32'h80000000,
	parameter C_HIGHADDR = 32'h800000FF,
	parameter C_OPB_AWIDTH = 32,
	parameter C_OPB_DWIDTH = 32,
	parameter C_USER_ID_CODE = 3,
	parameter C_FAMILY = "virtex2p"
)
(
	// OPB Bus Signals
	input								OPB_Clk,
	input								OPB_Rst,
	output reg [C_OPB_DWIDTH-1:0]	Sl_DBus,
	output							Sl_errAck,
	output							Sl_retry,
	output reg						Sl_toutSup,
	output reg						Sl_xferAck,
	input [C_OPB_AWIDTH-1:0]	OPB_ABus,
	input [C_OPB_DWIDTH/8-1:0] OPB_BE,
	input [C_OPB_DWIDTH-1:0]	OPB_DBus,
	input								OPB_RNW,
	input								OPB_select,
	input								OPB_seqAddr,
	output							IP2INTC_Irpt,
	 
	input								clk75_n,
	input								clk75_p,
	
	output 							VGA_VSYNCH,
	output 							VGA_HSYNCH,
	output 							VGA_OUT_BLANK_Z,
	output 							VGA_COMP_SYNCH,
	output 							VGA_OUT_PIXEL_CLOCK,
	
	output [7:0] 					VGA_OUT_RED,
	output [7:0] 					VGA_OUT_GREEN,
	output [7:0] 					VGA_OUT_BLUE
);

assign Sl_errAck = 1'b0;
assign Sl_retry = 1'b0;

//reg clk1; //wire clk1;
wire clk1;
reg clk1_n;
wire clk5;
wire clk25;
NESclks clkgen
(
	.clk75_n(clk75_n),
	.clk75_p(clk75_p),
	.clk1(),
	.clk25(clk25),
	.clk5(clk5)
);


assign clk1=clk5;
//reg [1:0] state_clk;
//localparam S1 = 2'd0;
//localparam S2 = 2'd1;
//localparam S3 = 2'd2;

//always @ (posedge clk5)
//begin
//	if (OPB_Rst) begin
//		state_clk <= S1;
//		clk1 <= 1'b0;
//	end
//	else begin
//		case(state_clk)
//		  S1:  state_clk <= S3;
//		  S2:  state_clk <= S3;
//		  S3:  begin
//		         state_clk <= S1;
//					if (clk1) begin
//					  clk1 <= 1'b0;
//					end
//					else begin
//					  clk1 <= 1'b1;
//					end 
//				 end
//		  default: state_clk <= S1;
//		endcase
//	end
//end

reg [11:0] fifoA_din;
wire [11:0] fifoA_dout;
reg fifoA_read;
reg fifoA_write;
wire fifoA_empty;
wire fifoA_full;

ppubridge_fifo fifo_opb_to_ppu
(
	.din(fifoA_din),
	.rd_clk(clk1),
	.rd_en(fifoA_read),
	.rst(OPB_Rst),
	.wr_clk(OPB_Clk),
	.wr_en(fifoA_write),
	.dout(fifoA_dout),
	.empty(fifoA_empty),
	.full(fifoA_full)
);

reg [11:0] fifoB_din;
wire [11:0] fifoB_dout;
reg fifoB_read;
reg fifoB_write;
wire fifoB_empty;
wire fifoB_full;

ppubridge_fifo fifo_ppu_to_opb
(
	.din(fifoB_din),
	.rd_clk(OPB_Clk),
	.rd_en(fifoB_read),
	.rst(OPB_Rst),
	.wr_clk(clk1),
	.wr_en(fifoB_write),
	.dout(fifoB_dout),
	.empty(fifoB_empty),
	.full(fifoB_full)
);


localparam S1_IDLE = 2'd0;
localparam S1_TRANSACTION = 2'd1;
localparam S1_ACK = 2'd2;

reg [1:0] state1, nextstate1;

always @ (posedge OPB_Clk or posedge OPB_Rst) begin
	if (OPB_Rst) begin
		state1 <= S1_IDLE;
	end
	else begin
		state1 <= nextstate1;
	end
end

always @* begin
	nextstate1 = state1;
	fifoA_write = 1'b0;
	fifoA_din = 12'bx;
	fifoB_read = 1'b0;
	Sl_toutSup = 1'b0;
	Sl_xferAck = 1'b0;
	Sl_DBus = 32'b0;
	
	case (state1)
		S1_IDLE : begin
			if (OPB_select && OPB_ABus >= C_BASEADDR && OPB_ABus <= C_HIGHADDR) begin
				Sl_toutSup = 1'b1;
				fifoA_din = {OPB_RNW, OPB_ABus[2:0], OPB_DBus[7:0]};
				if (fifoA_empty && fifoB_empty) begin
					fifoA_write = 1'b1;
					nextstate1 = S1_TRANSACTION;
				end
			end
		end
		
		S1_TRANSACTION : begin
			Sl_toutSup = 1'b1;
			if (!fifoB_empty) begin
				nextstate1 = S1_ACK;
			end
		end
		
		S1_ACK : begin
			fifoB_read = 1'b1;
			Sl_xferAck = 1'b1;
			if (OPB_RNW) begin
				Sl_DBus = fifoB_dout[7:0];
			end
			nextstate1 = S1_IDLE;
		end
	endcase
end
				

	



// PPU instantiation

reg [2:0] ppu_addr;
reg [7:0] ppu_write_data;
wire [7:0] ppu_read_data;
reg ppu_read;
reg ppu_write;
wire [5:0] ppu_pixel;
wire ppu_pixel_valid;
wire [7:0] ppu_pixel_x;
wire [7:0] ppu_pixel_y;
ppu the_ppu
(
	.i_clk_54(clk5),
	.i_clk_17(clk1), 
	.i_reset(OPB_Rst),
	
	.i_addr(ppu_addr),
	.i_wr_data(ppu_write_data),
	.o_rd_data(ppu_read_data),
	.i_read(ppu_read),
	.i_write(ppu_write),
	
	.o_nmi(IP2INTC_Irpt),
	
	.o_pixel(ppu_pixel),
	.o_pixel_valid(ppu_pixel_valid),
	.o_pixel_x(ppu_pixel_x),
	.o_pixel_y(ppu_pixel_y)
);


always @* begin
	ppu_read = !fifoA_empty && fifoA_dout[11];
	ppu_write = !fifoA_empty && ~fifoA_dout[11];
	ppu_write_data = fifoA_dout[7:0];
	ppu_addr = fifoA_dout[10:8];
	fifoA_read = !fifoA_empty;
	fifoB_write = !fifoA_empty;
	fifoB_din = {4'bx, ppu_read_data};
end
	

vid_out vid_out_inst
(
	.clk25(clk25),
	.clk5(clk5),
	
	.pixel_colour(ppu_pixel),
	.colour_emphasis(2'b0),
	.reset(OPB_Rst),
	.write_pixel(ppu_pixel_valid),
	.pixel_x(ppu_pixel_x),
	.pixel_y(ppu_pixel_y),
	
	.VGA_VSYNCH(VGA_VSYNCH),
	.VGA_HSYNCH(VGA_HSYNCH),
	.VGA_OUT_BLANK_Z(VGA_OUT_BLANK_Z),
	.VGA_COMP_SYNCH(VGA_COMP_SYNCH),
	.VGA_OUT_PIXEL_CLOCK(VGA_OUT_PIXEL_CLOCK),
	
	.VGA_OUT_RED(VGA_OUT_RED),
	.VGA_OUT_GREEN(VGA_OUT_GREEN),
	.VGA_OUT_BLUE(VGA_OUT_BLUE)
);

endmodule
