`timescale 1ns / 1ps

module regs_tb_v;

	// Inputs
	reg i_clk;
	reg i_reset;
	reg [2:0] i_cpu_addr;
	reg i_cpu_write;
	reg i_cpu_read;
	reg [7:0] i_cpu_write_data;
	wire [7:0] i_vram_read_data;
	wire [7:0] i_pal_rd_data;
	wire [7:0] i_spr_rd_data;
	reg i_hscroll_cnt_load;
	reg i_all_cnt_load;
	reg i_hscroll_cnt_enable;
	reg i_vscroll_cnt_enable;
	reg i_rend_nt_read;
	reg i_rend_at_read;
	reg i_rend_pt_read;
	reg i_rend_pt_bit3;
	reg i_rend_set_vblank;
	reg i_rend_clear_vblank;
	reg i_rend_spritelimit;
	reg i_rend_primaryhit;

	// Outputs
	wire o_vblank_nmi;
	wire [7:0] o_cpu_read_data;
	wire o_cpu_datavalid;
	wire [13:0] o_vram_addr;
	wire o_vram_read;
	wire o_vram_write;
	wire [7:0] o_vram_write_data;
	wire [4:0] o_pal_addr;
	wire [7:0] o_pal_wr_data;
	wire o_pal_write;
	wire [7:0] o_spr_addr;
	wire [7:0] o_spr_wr_data;
	wire o_spr_write;
	wire [1:0] o_attrib;
	wire [2:0] o_fine_h;
	wire o_object_pat_table;
	wire o_object_big;
	wire o_object_clip;
	wire o_object_enable;
	wire o_pf_clip;
	wire o_pf_enable;

	vidmem the_vidmem
	(
		.i_clk(i_clk),
		.i_addr(o_vram_addr),
		.i_wr_data(o_vram_write_data),
		.i_write(o_vram_write),
		.i_read(o_vram_read),
		.o_rd_data(i_vram_read_data)
	);
	
	spr_ram the_spr_ram
	(
		.i_clk(i_clk),
		.i_cpu_addr(o_spr_addr),
		.i_cpu_write(o_spr_write),
		.i_cpu_wr_data(o_spr_wr_data),
		.o_cpu_rd_data(i_spr_rd_data),
		.i_rend_addr('d0)
	);
	
	palette the_pal
	(
		.i_clk(i_clk),
		.i_cpu_addr(o_pal_addr),
		.i_cpu_wr_data(o_pal_wr_data),
		.i_cpu_write(o_pal_write),
		.o_cpu_rd_data(i_pal_rd_data),
	
		.i_rend_addr('d0)
	);

	// Instantiate the Unit Under Test (UUT)
	regs uut (
		.i_clk(i_clk), 
		.i_reset(i_reset), 
		.o_vblank_nmi(o_vblank_nmi), 
		.i_cpu_addr(i_cpu_addr), 
		.i_cpu_write(i_cpu_write), 
		.i_cpu_read(i_cpu_read), 
		.i_cpu_write_data(i_cpu_write_data), 
		.o_cpu_read_data(o_cpu_read_data), 
		.o_cpu_datavalid(o_cpu_datavalid), 
		.o_vram_addr(o_vram_addr), 
		.o_vram_read(o_vram_read), 
		.o_vram_write(o_vram_write), 
		.i_vram_read_data(i_vram_read_data), 
		.o_vram_write_data(o_vram_write_data), 
		.o_pal_addr(o_pal_addr), 
		.o_pal_wr_data(o_pal_wr_data), 
		.o_pal_write(o_pal_write), 
		.i_pal_rd_data(i_pal_rd_data), 
		.o_spr_addr(o_spr_addr), 
		.o_spr_wr_data(o_spr_wr_data), 
		.o_spr_write(o_spr_write), 
		.i_spr_rd_data(i_spr_rd_data), 
		.i_hscroll_cnt_load(i_hscroll_cnt_load), 
		.i_all_cnt_load(i_all_cnt_load), 
		.i_hscroll_cnt_enable(i_hscroll_cnt_enable), 
		.i_vscroll_cnt_enable(i_vscroll_cnt_enable), 
		.i_rend_nt_read(i_rend_nt_read), 
		.i_rend_at_read(i_rend_at_read), 
		.i_rend_pt_read(i_rend_pt_read), 
		.i_rend_pt_bit3(i_rend_pt_bit3), 
		.o_attrib(o_attrib), 
		.o_fine_h(o_fine_h), 
		.i_rend_set_vblank(i_rend_set_vblank), 
		.i_rend_clear_vblank(i_rend_clear_vblank), 
		.i_rend_spritelimit(i_rend_spritelimit), 
		.i_rend_primaryhit(i_rend_primaryhit), 
		.o_object_pat_table(o_object_pat_table), 
		.o_object_big(o_object_big), 
		.o_object_clip(o_object_clip), 
		.o_object_enable(o_object_enable), 
		.o_pf_clip(o_pf_clip), 
		.o_pf_enable(o_pf_enable)
	);

	always #185 i_clk = ~i_clk;

	initial begin
		// Initialize Inputs
		i_clk = 0;
		i_reset = 0;
		i_cpu_addr = 0;
		i_cpu_write = 0;
		i_cpu_read = 0;
		i_cpu_write_data = 0;
		i_hscroll_cnt_load = 0;
		i_all_cnt_load = 0;
		i_hscroll_cnt_enable = 0;
		i_vscroll_cnt_enable = 0;
		i_rend_nt_read = 0;
		i_rend_at_read = 0;
		i_rend_pt_read = 0;
		i_rend_pt_bit3 = 0;
		i_rend_set_vblank = 0;
		i_rend_clear_vblank = 0;
		i_rend_spritelimit = 0;
		i_rend_primaryhit = 0;

		// Wait 100 ns for global reset to finish
		#100;
        
		// Add stimulus here

	end
      
endmodule

