`timescale 1ns / 1ps

module spr_buffer
(
	input i_clk,
	input i_reset,
	
	input i_load,
	input i_enable,
	
	input [7:0] i_pt0,
	input [7:0] i_pt1,
	input [1:0] i_pal,
	input [7:0] i_sprite_x,
	input i_bgpri,
	input i_spr0,
	
	output [3:0] o_pixel,
	output o_bgpri,
	output o_spr0
);

reg [15:0] sr_pt0;
reg [15:0] sr_pt1;
reg [8:0] sr_at0;
reg [8:0] sr_at1;
reg [7:0] x_cnt;
reg bgpri;
reg spr0;

wire counted_down = x_cnt == 8'd0;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		sr_pt0 <= 'd0;
		sr_pt1 <= 'd0;
		sr_at0 <= 'd0;
		sr_at1 <= 'd0;
		x_cnt <= 'd0;
		bgpri <= 'd0;
		spr0 <= 'd0;
	end
	else if (i_load) begin
		sr_pt0 <= {8'b0, i_pt0};
		sr_pt1 <= {8'b0, i_pt1};
		sr_at0 <= {8'b0, i_pal[0]};
		sr_at1 <= {8'b0, i_pal[1]};
		x_cnt <= i_sprite_x;
		bgpri <= i_bgpri;
		spr0 <= i_spr0;
	end
	else if (i_enable) begin
		if (counted_down) begin
			sr_pt0 <= {sr_pt0[14:0], 1'b0};
			sr_pt1 <= {sr_pt1[14:0], 1'b0};
			sr_at0 <= {sr_at0[7:0], sr_at0[0]};
			sr_at1 <= {sr_at1[7:0], sr_at1[0]};
		end
		else begin
			x_cnt <= x_cnt - 8'd1;
		end
	end
end

assign o_bgpri = bgpri;
assign o_spr0 = spr0;
assign o_pixel = {sr_at1[8], sr_at0[8], sr_pt1[15], sr_pt0[15]};
			

endmodule
