`timescale 1ns / 1ps

module spr_evaluator
(
	input i_clk,
	input i_reset,
	
	input i_data_valid,
	input [7:0] i_cur_y,
	input [7:0] i_spr_x_coord,	
	input [7:0] i_spr_y_coord,
	input [7:0] i_spr_tilenum,
	input [7:0] i_spr_attr,
	input [5:0] i_spr_index,
	
	input i_cnt_reset,
	output [3:0] o_num_sprites,
	
	output o_tmpmem_write,
	output [2:0] o_tmpmem_addr,
	output [24:0] o_tmpmem_data,
	
	input i_object_big,
	output o_set_spritelimit
);

// Sprite counter

reg [3:0] addr_counter;
wire addr_cnt_enable;
always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		addr_counter <= 'd0;
	end
	else if (i_cnt_reset) begin
		addr_counter <= 'd0;
	end
	else if (addr_cnt_enable) begin
		if (addr_counter != 'd8) addr_counter <= addr_counter + 4'd1;
	end
end

assign o_set_spritelimit = addr_counter[3];
assign o_num_sprites = addr_counter;

// In-range evaluator

wire [8:0] compare_result = i_cur_y - i_spr_y_coord;
wire in_range = ~compare_result[8] & (i_object_big? compare_result < 16 : compare_result < 8);
wire spr0_found = in_range && (i_spr_index == 6'd0);

assign addr_cnt_enable = in_range & i_data_valid;

// Output to sprite temporary memory

wire [3:0] inverted_result = i_spr_attr[7]? ~compare_result[3:0] : compare_result[3:0];

assign o_tmpmem_write = ~addr_counter[3] & in_range & i_data_valid;
assign o_tmpmem_addr = addr_counter[2:0];
assign o_tmpmem_data = {spr0_found, i_spr_attr[6], i_spr_attr[5], i_spr_attr[1:0],
								inverted_result, i_spr_x_coord, i_spr_tilenum};

endmodule
