`timescale 1ns / 1ps


module spr_fetcher
(
	input i_clk,
	input i_reset,
	
	output reg [2:0] o_tmpmem_addr,
	input [24:0] i_tmpmem_data,
	
	output reg o_pt_read,
	output reg o_pt_bit3,
	output reg o_pt_ptab,
	output reg [2:0] o_pt_fv,
	output reg [7:0] o_pt_tilenum,
	input [7:0] i_pt_data,
	
	input i_go,
	input [3:0] i_num_sprites,
	input i_object_big,
	input i_object_ptab,
	
	output reg [7:0] o_sprbuf_load,
	output reg [7:0] o_sprbuf_p0,
	output reg [7:0] o_sprbuf_p1,
	output reg [7:0] o_sprbuf_x,
	output reg [3:0] o_sprbuf_attr
);


function [7:0] reverse_bits;
	input [7:0] x;
	begin
		reverse_bits = {x[0], x[1], x[2], x[3], x[4], x[5], x[6], x[7]};
	end
endfunction

wire [7:0] data_tile = i_tmpmem_data[7:0];
wire [7:0] data_x = i_tmpmem_data[15:8];
wire [3:0] data_range = i_tmpmem_data[19:16];
wire [3:0] data_attr = i_tmpmem_data[23:20];
wire data_spr0 = i_tmpmem_data[24];

reg [2:0] state;
localparam S_IDLE = 3'd0;
localparam S_READ_SPR = 3'd1;
localparam S_READ_P0 = 3'd2;
localparam S_READ_P1 = 3'd3;
localparam S_WRITE_BUF = 3'd4;

reg [7:0] spr_pat0;
reg [2:0] sprite_counter;
reg [3:0] num_sprites;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		state <= S_IDLE;
	end
	else begin
		case (state)
			S_IDLE: begin
				sprite_counter <= 'd0;
				num_sprites <= i_num_sprites;
				if (i_go) begin	
					state <= S_READ_SPR;
				end
			end
			
			S_READ_SPR: begin
				state <= S_READ_P0;
			end
			
			S_READ_P0: begin
				state <= S_READ_P1;
			end
			
			S_READ_P1: begin
				state <= S_WRITE_BUF;
				spr_pat0 <= data_attr[3] ? reverse_bits(i_pt_data) : i_pt_data;
			end
			
			S_WRITE_BUF: begin
				sprite_counter <= sprite_counter + 3'd1;
				
				if (sprite_counter == 3'd7) begin
					state <= S_IDLE;
				end
				else begin
					state <= S_READ_SPR;
				end
			end
		endcase
	end
end



reg [2:0] sprbuf_addr;
reg sprbuf_write;

always @* begin
	o_pt_read = 1'b0;
	o_pt_bit3 = 1'bx;
	o_tmpmem_addr = sprite_counter;
	o_pt_fv = data_range[2:0];
	
	sprbuf_write = 1'b0;
	sprbuf_addr = 'dx;
	o_sprbuf_p0 = 'dx;
	o_sprbuf_p1 = 'dx;
	o_sprbuf_x = 'dx;
	o_sprbuf_attr = 'dx;
	
	if (i_object_big) begin
		o_pt_tilenum = {data_tile[7:1], data_range[3]};
		o_pt_ptab = data_tile[0];
	end
	else begin
		o_pt_tilenum = data_tile;
		o_pt_ptab = i_object_ptab;
	end
	
	case (state)
		S_READ_SPR: begin
			o_tmpmem_addr = sprite_counter;
		end
		
		S_READ_P0: begin
			o_pt_read = 1'b1;
			o_pt_bit3 = 1'b0;
		end
		
		S_READ_P1: begin
			o_pt_read = 1'b1;
			o_pt_bit3 = 1'b1;
		end
		
		S_WRITE_BUF: begin
			sprbuf_write = 1'b1;
			sprbuf_addr = sprite_counter;
			o_sprbuf_x = data_x;
			
			if (sprite_counter < num_sprites) begin
				o_sprbuf_p0 = spr_pat0;
				o_sprbuf_p1 = data_attr[3] ? reverse_bits(i_pt_data) : i_pt_data;
				o_sprbuf_attr = {data_spr0, data_attr[2:0]};
			end
			else begin
				o_sprbuf_p0 = 'd0;
				o_sprbuf_p1 = 'd0;
				o_sprbuf_attr = 'd0;
			end
		end
	endcase
end

always @* begin
	case (sprbuf_addr)
		3'd0 : o_sprbuf_load = {7'b0, sprbuf_write};
		3'd1 : o_sprbuf_load = {6'b0, sprbuf_write, 1'b0};
		3'd2 : o_sprbuf_load = {5'b0, sprbuf_write, 2'b0};
	   3'd3 : o_sprbuf_load = {4'b0, sprbuf_write, 3'b0};
		3'd4 : o_sprbuf_load = {3'b0, sprbuf_write, 4'b0};
		3'd5 : o_sprbuf_load = {2'b0, sprbuf_write, 5'b0};
		3'd6 : o_sprbuf_load = {1'b0, sprbuf_write, 6'b0};
		3'd7 : o_sprbuf_load = {sprbuf_write, 7'b0};
		default : o_sprbuf_load = 'b0;
	endcase
end
		
endmodule
